# -*- coding: utf-8 -*-
"""
This package contains all of the modules utilised by the creds library.

constants: Functions to define and discover OS constants.

plan: Functions to generate a list of steps to transition from the current state to the desired state.

ssh: Contains a class to represent a users' keys and functions to manage them.

users: Functions  to managing users and classes...
 Users - representation of a user list and methods to read, write and manage them
 User - representation of a single user with their associated credentials

utils: Common helper functions.
"""

from __future__ import (unicode_literals, print_function)

__title__ = 'creds'
__version__ = '1.0.16'
__author__ = 'Jon Hadfield'
__license__ = 'MIT'
__copyright__ = 'Copyright 2016 Jon Hadfield'
